﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Collections.Generic;

using Generators;

namespace FractalGenerator
{
    public partial class FractalGenerator : Form
    {
        private List<IGenerator> generators;
        private Options preferences;
        
        public FractalGenerator()
        {
            this.generators = new List<IGenerator>();
            this.generators.Add(new Mandelbrot());
            this.generators.Add(new MandelbrotEscapeTime());
            this.generators.Add(new MandelbrotBW());
            this.generators.Add(new JuliaEscapeTime());
            this.generators.Add(new Newton3());
            this.generators.Add(new Newton3BW());
            this.generators.Add(new BDOLM());

            this.preferences = new Options(400, 400, 2, 4, 5);

            InitializeComponent();
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            foreach (IGenerator g in this.generators)
            {
                ToolStripItem item = new ToolStripButton(g.GetName());
                item.Tag = g;
                item.Click += new EventHandler(generator_Click);
                item.AutoSize = false;
                item.Width = 100;
                item.TextAlign = ContentAlignment.MiddleLeft;

                ToolStripMenuItem menu = (ToolStripMenuItem)this.mainMenu.Items["generatorsMenuItem"];
                menu.DropDownItems.Add(item);
            }
        }

        void generator_Click(object sender, EventArgs e)
        {
            IGenerator generator = ((IGenerator)((ToolStripItem)sender).Tag).CreateGenerator();

            FractalView newForm = new FractalView(generator.GetName(), generator, this.preferences);
            newForm.MdiParent = this;

            newForm.Show();
        }

        private void preferencesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OptionsDialog dlg = new OptionsDialog(this.preferences);
            if (dlg.ShowDialog(this) == DialogResult.OK)
            {
                this.preferences = dlg.Preferences;
            }
        }
    }
}
